searchData={"items":[{"type":"module","title":"EctoI18n","doc":"Provides i18n support for Ecto.","ref":"EctoI18n.html"},{"type":"module","title":"Preface - EctoI18n","doc":"There're [lots of strategies to localize contents in database](https://dejimata.com/2017/3/3/translating-with-mobility)\n([archived](https://web.archive.org/web/20240528023514/https://dejimata.com/2017/3/3/translating-with-mobility)).\nFor now, `EctoI18n` implements only strategy 6 mentioned\nabove - using a column of `:map` type for storing all the related localized data.\n\nWith this strategy, it can:\n\n  * avoid using extra tables for storing localized data.\n  * avoid using complex JOINs when retrieving localized data.\n\n> Maybe other strategies will be implemented later, but for now, I only\n> need this one.","ref":"EctoI18n.html#module-preface"},{"type":"module","title":"Quick start - EctoI18n","doc":"Let's say that we have a schema which needs to be localized:\n\n    defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n\n      schema \"products\" do\n        field :sku, :string\n        field :name, :string\n      end\n    end\n\nThe first step is to add a new column to the table at database level, so\nwe can store localized data in it:\n\n    defmodule MyApp.Repo.Migrations.AddLocalesToProducts do\n      use Ecto.Migration\n\n      def change do\n        alter table(:products) do\n          add :name_i18n, :map, null: false\n        end\n      end\n    end\n\nThe second step is to update schema for using the new column:\n\n    defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\"]\n\n      schema \"products\" do\n        field :sku, :string\n        field_i18n :name, :string\n      end\n    end\n\n> If you're curious about the underlying implementation here, you can read\n> `EctoI18n.Schema` to learn more.\n\nNext, you can use the extensions provided by `EctoI18n` to\nwork with the localized schema, such as:\n\n  * `EctoI18n.i18n_support?/1` / `EctoI18n.i18n_support?/2`\n  * `EctoI18n.locales/1`\n  * `EctoI18n.localize!/2`\n  * `EctoI18n.Changeset.cast_i18n/2` / `EctoI18n.Changeset.cast_i18n/3`\n  * `EctoI18n.Query` (Still in planning)\n  * ...","ref":"EctoI18n.html#module-quick-start"},{"type":"function","title":"EctoI18n.i18n_support?/1","doc":"Checks whether a module or a struct has i18n support.","ref":"EctoI18n.html#i18n_support?/1"},{"type":"function","title":"Examples - EctoI18n.i18n_support?/1","doc":"iex> EctoI18n.i18n_support?(Product)\n    iex> EctoI18n.i18n_support?(%Product{})","ref":"EctoI18n.html#i18n_support?/1-examples"},{"type":"function","title":"EctoI18n.i18n_support?/2","doc":"","ref":"EctoI18n.html#i18n_support?/2"},{"type":"function","title":"EctoI18n.locales/1","doc":"Returns supported locales of a struct or the underlying module.","ref":"EctoI18n.html#locales/1"},{"type":"function","title":"EctoI18n.localize!/2","doc":"Localizes a struct with given locale recursively.\n\nAll localizable values in the struct will be localized into the give locale.","ref":"EctoI18n.html#localize!/2"},{"type":"function","title":"Examples - EctoI18n.localize!/2","doc":"iex> EctoI18n.localize!(product, \"zh-Hans\")","ref":"EctoI18n.html#localize!/2-examples"},{"type":"type","title":"EctoI18n.locale/0","doc":"","ref":"EctoI18n.html#t:locale/0"},{"type":"module","title":"EctoI18n.Changeset","doc":"Provides i18n extensions for `Ecto.Changeset`.","ref":"EctoI18n.Changeset.html"},{"type":"function","title":"EctoI18n.Changeset.cast_i18n/3","doc":"Casts field which is added by `field_i18n/_` macro.\n\n> It is built on the top of `Ecto.Changeset.cast_embed/3`.","ref":"EctoI18n.Changeset.html#cast_i18n/3"},{"type":"function","title":"Example - EctoI18n.Changeset.cast_i18n/3","doc":"defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      import Ecto.Changeset\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\"]\n      import EctoI18n.Changeset\n\n      schema \"products\" do\n        field :sku, :string\n        field_i18n :name, :string\n      end\n\n      def changeset(product, attrs) do\n        product\n        |> cast(attrs, [:sku])\n        |> cast_i18n(:name, required: true)\n      end\n    end\n\nIn above code, `changeset/2` equals to:\n\n    def changeset(product, attrs) do\n      product\n      |> cast(attrs, [:sku])\n      |> cast_embed(:name_i18n, required: true)\n    end","ref":"EctoI18n.Changeset.html#cast_i18n/3-example"},{"type":"module","title":"EctoI18n.Query","doc":"Provides i18n extensions for `Ecto.Query`.","ref":"EctoI18n.Query.html"},{"type":"module","title":"Plans - EctoI18n.Query","doc":"I plan to build this module like `Cldr.Trans.QueryBuilder`.\n\n  * https://github.com/elixir-cldr/cldr_trans/blob/main/lib/cldr/trans/query_builder.ex\n  * https://github.com/elixir-cldr/cldr_trans/blob/main/lib/cldr/trans/gen_function_migration.ex","ref":"EctoI18n.Query.html#module-plans"},{"type":"module","title":"EctoI18n.Schema","doc":"Provides i18n extensions for `Ecto.Schema`.","ref":"EctoI18n.Schema.html"},{"type":"macro","title":"EctoI18n.Schema.field_i18n/3","doc":"Creates a virtual field and an embed for storing localized data.","ref":"EctoI18n.Schema.html#field_i18n/3"},{"type":"macro","title":"Example - EctoI18n.Schema.field_i18n/3","doc":"defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\"]\n\n      schema \"products\" do\n        field :sku, :string\n        field_i18n :name, :string\n      end\n    end\n\nIn above code, calling:\n\n    field_i18n :name, :string\n\nequals to:\n\n    field :name, :string, virtual: true\n    embeds_one :name_i18n, NameI18n do\n      field :\"en\", :string\n      field :\"zh-Hans\", :string\n    end\n\nThe `NameI18n` module will be generated automatically.","ref":"EctoI18n.Schema.html#field_i18n/3-example"},{"type":"extras","title":"EctoI18n","doc":"# EctoI18n\n\n> Provides i18n support for Ecto.","ref":"readme.html"},{"type":"extras","title":"Notes - EctoI18n","doc":"This package is still in its early stages, so it may still undergo significant changes, potentially leading to breaking changes.","ref":"readme.html#notes"},{"type":"extras","title":"Installation - EctoI18n","doc":"Add `:ecto_i18n` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_i18n,  }\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - EctoI18n","doc":"For more information, see the [documentation](https://hexdocs.pm/ecto_i18n).","ref":"readme.html#usage"},{"type":"extras","title":"Terminology - EctoI18n","doc":"Throughout the package, I will use the following terminology to avoid conceptual confusion.","ref":"readme.html#terminology"},{"type":"extras","title":"g11n (globalization) - EctoI18n","doc":"It is a broader concept that covers the i18n and l10n processes throughout the software development lifecycle.\n\nThe goal of g11n is to make products adapted in different languages and cultural environments:\n\n- i18n enables products to be adaptable.\n- l10n lets products to be adapted.","ref":"readme.html#g11n-globalization"},{"type":"extras","title":"i18n (internationalization) - EctoI18n","doc":"It refers to the process of designing and developing a product to be adaptable to different languages and cultural environments.\n\nIt is the preparation for l10n. When i18n is done, there's no need for further programmatic changes to the product to switch between languages.\n\nAt this stage, we are not conducting actual l10n work but rather providing the infrastructure for l10n.","ref":"readme.html#i18n-internationalization"},{"type":"extras","title":"l10n (localization) - EctoI18n","doc":"It refers to the process of adapting a product to a specific language or region. It includes:\n\n- translating text.\n- adjusting formats, such as date, time, number, currencies, etc.\n- ...\n\nAt this stage, we focus on the actual l10n work.","ref":"readme.html#l10n-localization"},{"type":"extras","title":"Thanks - EctoI18n","doc":"This package is built or will be built on the wisdom of:\n\n- [trasto](https://github.com/mjonuschat/trasto)\n- [trans](https://github.com/crbelaus/trans) / [cldr_trans](https://github.com/elixir-cldr/cldr_trans)\n- [i18n_helpers](https://github.com/mathieuprog/i18n_helpers)\n  - [Announcements at ElixirForum](https://elixirforum.com/t/i18n-helpers-ease-the-use-of-embedded-translations-in-ecto-schemas/25617)\n- [ecto_translate](https://github.com/smeevil/ecto_translate)","ref":"readme.html#thanks"},{"type":"extras","title":"License - EctoI18n","doc":"[Apache License 2.0](http://www.apache.org/licenses/LICENSE-2.0)","ref":"readme.html#license"},{"type":"extras","title":"Integrations","doc":"# Integrations","ref":"integrations.html"},{"type":"extras","title":"Phoenix LiveView - Integrations","doc":"","ref":"integrations.html#phoenix-liveview"},{"type":"extras","title":"Form - Integrations","doc":"Take `EctoI18n.Product` schema located at `test/support/schemas.ex` as an example. If you want to create a form for it, try something like:\n\n```heex\n<.input field={@form[:sku]} type=\"text\" label=\"SKU\" />\n<.inputs_for :let={name} field={@form[:name_i18n]}>\n  <%= for locale <- EctoI18n.locales(@form.data) do %>\n    <.input field={name[locale]} type=\"text\" label={\"Name(#{locale})\"} />\n  <% end %>\n</.inputs_for>\n<.inputs_for :let={price} field={@form[:price_i18n]}>\n  <%= for locale <- EctoI18n.locales(@form.data) do %>\n    <.input field={price[locale]} type=\"number\" label={\"Price(#{locale})\"} />\n  <% end %>\n</.inputs_for>\n```","ref":"integrations.html#form"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}